// This macro setup.C is called in TXXXAnalysis
#include "Riostream.h"
void setup()
{
  TGo4AnalysisStep        * step;
  TGo4StepFactory         * fact;
  TGo4FileStoreParameter  * store;
  TGo4FileSourceParameter * input;

  Text_t autosave[]="result";      // name of autosave file (.root)
  Text_t Unp1Out[] ="unpacked1";   // name of output file step 1 (.root)
  Text_t Cal1Out[] ="calibrated1"; // name of output file step 2 (.root)
  Text_t Unp2Out[] ="unpacked2";   // name of output file step 3 (.root)
  Text_t Cal2Out[] ="calibrated2"; // name of output file step 4 (.root)
  Text_t PhysOut[] ="physics";     // name of output file step 5 (.root)
  Text_t node[]="nodename";        // name of MBS or event server node
  Int_t  port=0;                   // depends on the remote event server

  cout << "**** Setup steps" << endl;
  go4->SetAutoSaveFile(autosave); // optional
  go4->SetAutoSaveInterval(0);    // after n seconds , 0 = at end of event loop   
  go4->SetAutoSave(kTRUE);       // optional

  // some processors do not use output of previous as input.
  // Therefore we must switch off step checking.
  // We must be sure that all processor get their input event properly
  // In your case means that:
  // MakePhys needs output events of Cal1 and Cal2, and
  // Unpack2 need input event of Unpack1

  go4->SetStepChecking(kFALSE);

// Analysis has the step, which has the factory, 
// which handles the event processor and event objects

  TGo4StepFactory*  facUnp1   = new TGo4StepFactory("XXXUnp1");
  TGo4AnalysisStep* stepUnp1  = new TGo4AnalysisStep("Unpack1",facUnp1,0,0,0);
  go4->AddAnalysisStep(stepUnp1);
  TGo4StepFactory*  facCal1   = new TGo4StepFactory("XXXCal1");
  TGo4AnalysisStep* stepCal1  = new TGo4AnalysisStep("Cal1",facCal1,0,0,0);
  go4->AddAnalysisStep(stepCal1);
  TGo4StepFactory*  facUnp2   = new TGo4StepFactory("XXXUnp2");
  TGo4AnalysisStep* stepUnp2  = new TGo4AnalysisStep("Unpack2",facUnp2,0,0,0);
  go4->AddAnalysisStep(stepUnp2);
  TGo4StepFactory*  facCal2   = new TGo4StepFactory("XXXCal2");
  TGo4AnalysisStep* stepCal2  = new TGo4AnalysisStep("Cal2",facCal2,0,0,0);
  go4->AddAnalysisStep(stepCal2);
  TGo4StepFactory*  facPhys  = new TGo4StepFactory("XXXPhys");
  TGo4AnalysisStep* stepPhys = new TGo4AnalysisStep("Physics",facPhys,0,0,0);
  go4->AddAnalysisStep(stepPhys);

  //=========================================================
  // Set up step Unpack1 subevent 1
  //=========================================================
  step = go4->GetAnalysisStep("Unpack1");
  fact=(TGo4StepFactory *)step->GetStepFactory();
  fact->DefEventProcessor("XXXMakeUnp1","TXXXMakeUnp1");
  fact->DefOutputEvent("XXXEventUnp1","TXXXEventUnp1");

  // activate one of the following MBS event sources:
  //--------------------------------------------------
  TGo4MbsRandomParameter * source = new TGo4MbsRandomParameter("Random");
  //--------------------------------------------------
  // TGo4MbsEventServerParameter * source = new TGo4MbsEventServerParameter(node);
  //--------------------------------------------------
  // TGo4MbsStreamParameter * source = new TGo4MbsStreamParameter(node);
  //--------------------------------------------------
  // TGo4MbsTransportParameter * source = new TGo4MbsTransportParameter(node);
  //--------------------------------------------------
  // TGo4MbsFileParameter * source = new TGo4MbsFileParameter(node);
  //--------------------------------------------------
  // TGo4RevServParameter * source=new TGo4RevServParameter(node);
  // source->SetPort(port);
  //--------------------------------------------------

  step->SetEventSource(source);  // register event source
  step->SetSourceEnabled(kTRUE);
  step->SetProcessEnabled(kTRUE);
  step->SetErrorStopEnabled(kTRUE);
  store = new TGo4FileStoreParameter(Unp1Out);
  store->SetOverwriteMode(kTRUE);
  step->SetEventStore(store);
  step->SetStoreEnabled(kFALSE);  // en-disable output

  //=========================================================
  // Set up step Calibrate1 subcrate 1
  //=========================================================
  step = go4->GetAnalysisStep("Cal1");
  fact=(TGo4StepFactory *)step->GetStepFactory();
  fact->DefInputEvent("XXXEventUnp1","TXXXEventUnp1");
  fact->DefEventProcessor("XXXMakeCal1","TXXXMakeCal1");
  fact->DefOutputEvent("XXXEventCal1","TXXXEventCal1");
  input = new TGo4FileSourceParameter(Unp1Out);
  step->SetEventSource(input);
  step->SetSourceEnabled(kFALSE); //use input
  step->SetProcessEnabled(kTRUE);
  step->SetErrorStopEnabled(kTRUE);
  store = new TGo4FileStoreParameter(Cal1Out);
  store->SetOverwriteMode(kTRUE);
  step->SetEventStore(store);
  step->SetStoreEnabled(kFALSE);  // en-disable output

  //=========================================================
  // Set up step Unpack2 subevent 2, source from step 1
  //=========================================================
  step = go4->GetAnalysisStep("Unpack2");
  fact=(TGo4StepFactory *)step->GetStepFactory();
  fact->DefEventProcessor("XXXMakeUnp2","TXXXMakeUnp2");
  fact->DefOutputEvent("XXXEventUnp2","TXXXEventUnp2");
  step->SetSourceEnabled(kFALSE); // use input as step 1
  step->SetProcessEnabled(kTRUE);
  step->SetErrorStopEnabled(kTRUE);
  store = new TGo4FileStoreParameter(Unp2Out);
  store->SetOverwriteMode(kTRUE);
  step->SetEventStore(store);
  step->SetStoreEnabled(kFALSE);  // en-disable output

  //=========================================================
  // Set up step Calibrate2 subcrate 2
  //=========================================================
  step = go4->GetAnalysisStep("Cal2");
  fact=(TGo4StepFactory *)step->GetStepFactory();
  fact->DefInputEvent("XXXEventUnp2","TXXXEventUnp2");
  fact->DefEventProcessor("XXXMakeCal2","TXXXMakeCal2");
  fact->DefOutputEvent("XXXEventCal2","TXXXEventCal2");
  input = new TGo4FileSourceParameter(Unp2Out);
  step->SetEventSource(input);
  step->SetSourceEnabled(kFALSE);
  step->SetProcessEnabled(kTRUE);
  step->SetErrorStopEnabled(kTRUE);
  store = new TGo4FileStoreParameter(Cal2Out);
  store->SetOverwriteMode(kTRUE);
  step->SetEventStore(store);
  step->SetStoreEnabled(kFALSE);  // en-disable output

  //=========================================================
  // Set up step Physics, get XXXEventCal1/2 in TXXXMakePhys
  //=========================================================
  step = go4->GetAnalysisStep("Physics");
  fact=(TGo4StepFactory *)step->GetStepFactory();
  fact->DefEventProcessor("XXXMakePhys","TXXXMakePhys");
  fact->DefOutputEvent("XXXEventPhys","TXXXEventPhys");
  step->SetSourceEnabled(kFALSE);
  step->SetProcessEnabled(kTRUE);
  step->SetErrorStopEnabled(kTRUE);
  store = new TGo4FileStoreParameter(PhysOut);
  store->SetOverwriteMode(kTRUE);
  step->SetEventStore(store);
  step->SetStoreEnabled(kFALSE);  // en-disable output

  cout << "**** Setup steps done" << endl;
}
